/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import java.util.Map;

import org.sablecc.sablecc.analysis.DepthFirstAdapter;
import org.sablecc.sablecc.node.*;

public class ConstructParserGenerationDatas extends DepthFirstAdapter
{
  private ResolveIds ids;
  private int currentAlt;
  private boolean processingAst;
  private String currentProd;

  private Map<String, Node> alts;

  public ConstructParserGenerationDatas(ResolveIds ids, Map<String, Node> alts)
  {
    this.ids = ids;
    this.alts = alts;
  }

  @Override
  public void caseAAst(AAst node)
  {}

  @Override
  public void caseAProd(AProd node)
  {
    currentProd = ids.names.get(node);
    for(PAlt alt : node.getAlts())
    {
      alt.apply(this);
    }
  }

  @Override
  public void caseAAlt(AAlt node)
  {
    currentAlt = Grammar.addProduction(currentProd, ids.names.get(node));
    alts.put(ids.names.get(node), node);

    for(PElem elem : node.getElems())
    {
      elem.apply(this);
    }
  }

  @Override
  public void caseAElem(AElem node)
  {
    String name = ResolveIds.name(node.getId().getText());

    if(node.getSpecifier() != null)
    {
      if(node.getSpecifier() instanceof ATokenSpecifier)
      {
        ids.elemTypes.put(node, "T" + name);
      }
      else
      {
        ids.elemTypes.put(node, "P" + name);
      }
    }
    else
    {
      ATokenDef token = ids.tokens.get("T" + name);

      if(token != null)
      {
        ids.elemTypes.put(node, "T" + name);
      }
      else
      {
        ids.elemTypes.put(node, "P" + name);
      }
    }
    name = ids.elemTypes.get(node);
    Grammar.addSymbolToProduction(name, currentAlt);
  }
}
